@@ -234,10 +234,11 @@ oddjob_dbus_listener_free(struct oddjob_dbus_context *ctx)
 	if (ctx->registered) {
 		dbus_connection_remove_filter(ctx->conn,
 					      oddjob_dbus_filter,
-					      NULL);
+					      ctx);
 		ctx->registered = FALSE;
 	}
-	oddjob_dbus_connection_close(ctx->conn);
+	/* Apparently we abort now when we try this. */
+	/* oddjob_dbus_connection_close(ctx->conn); */
 	ctx->conn = NULL;
 	oddjob_free(ctx);
 }
@@ -1098,7 +1099,7 @@ oddjob_dbus_listener_remove_method(struct oddjob_dbus_context *ctx,
 	if (ctx->registered) {
 		dbus_connection_remove_filter(ctx->conn,
 					      oddjob_dbus_filter,
-					      NULL);
+					      ctx);
 		ctx->registered = FALSE;
 	}
 
