@@ -60,8 +60,6 @@
 #include "oddjob_dbus.h"
 #include "util.h"
 
-#ident "$Id: oddjobd.c,v 1.46 2007/09/05 21:43:43 nalin Exp $"
-
 #define ODDJOB_INTROSPECTION_HEADER \
 "<!DOCTYPE node \n"\
 "          PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n"\
@@ -1958,8 +1956,16 @@ oddjobd_exec_method(struct oddjob_dbus_context *ctx,
 				_exit(-1);
 			}
 			if (getfilecon(method->argv[0], &helper_context) == -1) {
-				/* No label? */
-				exec_errno = errno;
+				switch (errno) {
+				/* Not there? */
+				case ENOENT:
+					exec_errno = errno;
+					break;
+				default:
+					/* No label? */
+					exec_errno = 0xfd;
+					break;
+				}
 				write(3, &exec_errno, 1);
 				_exit(-1);
 			}
@@ -2016,13 +2022,33 @@ oddjobd_exec_method(struct oddjob_dbus_context *ctx,
 	close(exec_status[0]);
 	/* If we actually got a byte, then it's an exec() errno. */
 	if (i == 1) {
+		const char *message;
 		close(task->stdin_fd);
 		close(task->stdout_fd);
 		close(task->stderr_fd);
 		waitpid(task->pid, &status, 0);
+		switch (exec_errno) {
+		case 0xff:
+			message = "error determining SELinux context of caller";
+			break;
+		case 0xfd:
+			message = "error reading SELinux file context of "
+				  "helper";
+			break;
+		case 0xfe:
+			message = "error determining helper execution SELinux "
+				  "context";
+			break;
+		case 0xfc:
+			message = "error setting helper execution SELinux "
+				  "context";
+			break;
+		default:
+			message = strerror(exec_errno);
+			break;
+		}
 		snprintf(buf, sizeof(buf),
-			 "Child signalled exec() error: %s.",
-			 strerror(exec_errno));
+			 "Child signalled exec() error: %s.", message);
 		if (globals.debug) {
 			fprintf(stderr, "%s\n", buf);
 		}
@@ -3057,8 +3083,5 @@ main(int argc, char **argv)
 		unlink(options.pidfile);
 	}
 
-	/* Last ditch effort. */
-	oddjob_dbus_connection_close(dbus_bus_get(options.bus, NULL));
-
 	return 0;
 }
